@@ -24,10 +24,12 @@ from debputy.architecture_support import (
     DpkgArchitectureBuildProcessValuesTable,
     dpkg_architecture_table,
 )
+from debputy.dh.dh_assistant import read_dh_addon_sequences
 from debputy.exceptions import DebputyRuntimeError
 from debputy.filesystem_scan import FSROOverlay
 from debputy.highlevel_manifest import HighLevelManifest
 from debputy.highlevel_manifest_parser import YAMLManifestParser
+from debputy.integration_detection import determine_debputy_integration_mode
 from debputy.packages import (
     SourcePackage,
     BinaryPackage,
@@ -36,6 +38,7 @@ from debputy.packages import (
 from debputy.plugin.api import VirtualPath
 from debputy.plugin.api.impl import load_plugin_features
 from debputy.plugin.api.feature_set import PluginProvidedFeatureSet
+from debputy.plugin.api.spec import DebputyIntegrationMode
 from debputy.substitution import (
     Substitution,
     VariableContext,
@@ -100,6 +103,7 @@ class CommandContext:
         self._requested_plugins: Optional[Sequence[str]] = None
         self._plugins_loaded = False
         self._dctrl_parser: Optional[DctrlParser] = None
+        self.debputy_integration_mode: Optional[DebputyIntegrationMode] = None
         self._dctrl_data: Optional[
             Tuple[
                 "SourcePackage",
@@ -288,6 +292,20 @@ class CommandContext:
         debian_control = self.debian_dir.get("control")
         return debian_control is not None
 
+    def resolve_integration_mode(self) -> DebputyIntegrationMode:
+        integration_mode = self.debputy_integration_mode
+        if integration_mode is None:
+            r = read_dh_addon_sequences(self.debian_dir)
+            bd_sequences, dr_sequences, _ = r
+            all_sequences = bd_sequences | dr_sequences
+            integration_mode = determine_debputy_integration_mode(all_sequences)
+            if integration_mode is None:
+                _error(
+                    "Cannot resolve the integration mode expected for this package. Is this package using `debputy`?"
+                )
+            self.debputy_integration_mode = integration_mode
+        return integration_mode
+
     def manifest_parser(
         self,
         *,
@@ -302,6 +320,7 @@ class CommandContext:
             manifest_path = self.parsed_args.debputy_manifest
         if manifest_path is None:
             manifest_path = os.path.join(self.debian_dir.fs_path, "debputy.manifest")
+        debian_dir = self.debian_dir
         return YAMLManifestParser(
             manifest_path,
             source_package,
@@ -311,6 +330,7 @@ class CommandContext:
             dctrl_parser.dpkg_arch_query_table,
             dctrl_parser.build_env,
             self.load_plugins(),
+            self.resolve_integration_mode(),
             debian_dir=self.debian_dir,
         )
 
