@@ -13,7 +13,9 @@ from typing import (
     Iterable,
 )
 
-from debputy.analysis.debian_dir import scan_debian_dir
+from debputy.analysis.analysis_util import flatten_ppfs
+from debputy.analysis.debian_dir import resolve_debhelper_config_files
+from debputy.dh.dh_assistant import extract_dh_compat_level
 from debputy.linting.lint_util import LintState
 from debputy.lsp.debputy_ls import DebputyLanguageServer
 from debputy.lsp.diagnostics import DiagnosticData
@@ -36,6 +38,7 @@ from debputy.lsp.lsp_features import (
     lsp_will_save_wait_until,
     lsp_format_document,
     LanguageDispatch,
+    lsp_text_doc_inlay_hints,
 )
 from debputy.lsp.lsp_generic_deb822 import (
     deb822_completer,
@@ -57,6 +60,7 @@ from debputy.lsp.text_util import (
     normalize_dctrl_field_name,
     LintCapablePositionCodec,
     te_range_to_lsp,
+    te_position_to_lsp,
 )
 from debputy.lsp.vendoring._deb822_repro import (
     Deb822FileElement,
@@ -86,7 +90,15 @@ from debputy.lsprotocol.types import (
     WillSaveTextDocumentParams,
     TextEdit,
     DocumentFormattingParams,
+    InlayHintParams,
+    InlayHint,
+    InlayHintLabelPart,
 )
+from debputy.packager_provided_files import (
+    PackagerProvidedFile,
+    detect_all_packager_provided_files,
+)
+from debputy.plugin.api.impl import plugin_metadata_for_debputys_own_plugin
 from debputy.util import detect_possible_typo
 
 try:
@@ -454,6 +466,69 @@ def _debian_control_folding_ranges(
     return deb822_folding_ranges(ls, params, _DCTRL_FILE_METADATA)
 
 
+@lsp_text_doc_inlay_hints(_LANGUAGE_IDS)
+def _doc_inlay_hint(
+    ls: "DebputyLanguageServer",
+    params: InlayHintParams,
+) -> Optional[List[InlayHint]]:
+    doc = ls.workspace.get_text_document(params.text_document.uri)
+    lint_state = ls.lint_state(doc)
+    deb822_file = lint_state.parsed_deb822_file_content
+    if not deb822_file:
+        return None
+    inlay_hints = []
+    stanzas = list(deb822_file)
+    if len(stanzas) < 2:
+        return None
+    source_stanza = stanzas[0]
+    source_stanza_pos = source_stanza.position_in_file()
+    for stanza_no, stanza in enumerate(deb822_file):
+        stanza_range = stanza.range_in_parent()
+        if stanza_no < 1:
+            continue
+        pkg_kvpair = stanza.get_kvpair_element("Package", use_get=True)
+        if pkg_kvpair is None:
+            continue
+
+        inlay_hint_pos_te = pkg_kvpair.range_in_parent().end_pos.relative_to(
+            stanza_range.start_pos
+        )
+        inlay_hint_pos = doc.position_codec.position_to_client_units(
+            lint_state.lines,
+            te_position_to_lsp(inlay_hint_pos_te),
+        )
+        stanza_def = _DCTRL_FILE_METADATA.classify_stanza(stanza, stanza_no)
+        for known_field in stanza_def.stanza_fields.values():
+            if not known_field.inherits_from_source or known_field.name in stanza:
+                continue
+
+            inherited_value = source_stanza.get(known_field.name)
+            if inherited_value is not None:
+                kvpair = source_stanza.get_kvpair_element(known_field.name)
+                value_range_te = kvpair.range_in_parent().relative_to(source_stanza_pos)
+                value_range = doc.position_codec.range_to_client_units(
+                    lint_state.lines,
+                    te_range_to_lsp(value_range_te),
+                )
+                inlay_hints.append(
+                    InlayHint(
+                        inlay_hint_pos,
+                        [
+                            InlayHintLabelPart(
+                                f"{known_field.name}: {inherited_value}\n",
+                                tooltip="Inherited from Source stanza",
+                                location=Location(
+                                    params.text_document.uri,
+                                    value_range,
+                                ),
+                            ),
+                        ],
+                    )
+                )
+
+    return inlay_hints
+
+
 def _paragraph_representation_field(
     paragraph: Deb822ParagraphElement,
 ) -> Deb822KeyValuePairElement:
@@ -640,9 +715,9 @@ def _diagnostics_for_paragraph(
         normalized_field_name_lc = normalize_dctrl_field_name(field_name_lc)
         known_field = known_fields.get(normalized_field_name_lc)
         field_value = stanza[field_name]
-        kvpair_position = kvpair.position_in_parent().relative_to(stanza_position)
+        kvpair_range_te = kvpair.range_in_parent().relative_to(stanza_position)
         field_range_te = kvpair.field_token.range_in_parent().relative_to(
-            kvpair_position
+            kvpair_range_te.start_pos
         )
         field_position_te = field_range_te.start_pos
         field_range_server_units = te_range_to_lsp(field_range_te)
@@ -722,7 +797,7 @@ def _diagnostics_for_paragraph(
                 kvpair,
                 stanza,
                 stanza_position,
-                kvpair_position,
+                kvpair_range_te,
                 lint_state,
                 field_name_typo_reported=field_name_typo_detected,
             )
@@ -961,7 +1036,7 @@ def _package_range_of_stanza(
     binary_stanzas: List[Tuple[Deb822ParagraphElement, TEPosition]],
 ) -> Iterable[Tuple[str, Optional[str], Range]]:
     for stanza, stanza_position in binary_stanzas:
-        kvpair = stanza.get_kvpair_element("Package")
+        kvpair = stanza.get_kvpair_element("Package", use_get=True)
         if kvpair is None:
             continue
         representation_field_range = kvpair.range_in_parent().relative_to(
@@ -974,38 +1049,111 @@ def _package_range_of_stanza(
         yield stanza["Package"], stanza.get("Architecture"), representation_field_range
 
 
+def _packaging_files(
+    lint_state: LintState,
+) -> Iterable[PackagerProvidedFile]:
+    source_root = lint_state.source_root
+    debian_dir = lint_state.debian_dir
+    binary_packages = lint_state.binary_packages
+    if (
+        source_root is None
+        or not source_root.has_fs_path
+        or debian_dir is None
+        or binary_packages is None
+    ):
+        return
+
+    dh_sequencer_data = lint_state.dh_sequencer_data
+    dh_sequences = dh_sequencer_data.sequences
+    is_debputy_package = (
+        "debputy" in dh_sequences
+        or "zz-debputy" in dh_sequences
+        or "zz_debputy" in dh_sequences
+        or "zz-debputy-rrr" in dh_sequences
+    )
+    feature_set = lint_state.plugin_feature_set
+    known_packaging_files = feature_set.known_packaging_files
+    static_packaging_files = {
+        kpf.detection_value: kpf
+        for kpf in known_packaging_files.values()
+        if kpf.detection_method == "path"
+    }
+    ignored_path = set(static_packaging_files)
+
+    if is_debputy_package:
+        all_debputy_ppfs = list(
+            flatten_ppfs(
+                detect_all_packager_provided_files(
+                    feature_set.packager_provided_files,
+                    debian_dir,
+                    binary_packages,
+                    allow_fuzzy_matches=True,
+                    detect_typos=True,
+                    ignore_paths=ignored_path,
+                )
+            )
+        )
+        for ppf in all_debputy_ppfs:
+            if ppf.path.path in ignored_path:
+                continue
+            ignored_path.add(ppf.path.path)
+            yield ppf
+
+    # FIXME: This should read the editor data, but dh_assistant does not support that.
+    dh_compat_level, _ = extract_dh_compat_level(cwd=source_root.fs_path)
+    if dh_compat_level is not None:
+        debputy_plugin_metadata = plugin_metadata_for_debputys_own_plugin()
+        dh_pkgfile_docs = {
+            kpf.detection_value: kpf
+            for kpf in known_packaging_files.values()
+            if kpf.detection_method == "dh.pkgfile"
+        }
+        (
+            all_dh_ppfs,
+            _,
+            _,
+        ) = resolve_debhelper_config_files(
+            debian_dir,
+            binary_packages,
+            debputy_plugin_metadata,
+            dh_pkgfile_docs,
+            dh_sequences,
+            dh_compat_level,
+            saw_dh=dh_sequencer_data.uses_dh_sequencer,
+            ignore_paths=ignored_path,
+        )
+        for ppf in all_dh_ppfs:
+            if ppf.path.path in ignored_path:
+                continue
+            ignored_path.add(ppf.path.path)
+            yield ppf
+
+
 def _detect_misspelled_packaging_files(
     lint_state: LintState,
     binary_stanzas_w_pos: List[Tuple[Deb822ParagraphElement, TEPosition]],
     diagnostics: List[Diagnostic],
 ) -> None:
-    debian_dir = lint_state.debian_dir
-    binary_packages = lint_state.binary_packages
-    if debian_dir is None or binary_packages is None:
-        return
-    all_pkg_file_data, _, _, _ = scan_debian_dir(
-        lint_state.plugin_feature_set,
-        binary_packages,
-        debian_dir,
-    )
     stanza_ranges = {
         p: (a, r)
         for p, a, r in _package_range_of_stanza(lint_state, binary_stanzas_w_pos)
     }
-
-    for pkg_file_data in all_pkg_file_data:
-        binary_package = pkg_file_data.get("binary-package")
-        explicit_package = pkg_file_data.get("pkgfile-explicit-package-name", True)
-        name_segment = pkg_file_data.get("pkgfile-name-segment")
-        stem = pkg_file_data.get("pkgfile-stem")
+    for ppf in _packaging_files(lint_state):
+        binary_package = ppf.package_name
+        explicit_package = ppf.uses_explicit_package_name
+        name_segment = ppf.name_segment is not None
+        stem = ppf.definition.stem
         if binary_package is None or stem is None:
             continue
-        declared_arch, diag_range = stanza_ranges.get(binary_package)
+        res = stanza_ranges.get(binary_package)
+        if res is None:
+            continue
+        declared_arch, diag_range = res
         if diag_range is None:
             continue
-        path = pkg_file_data["path"]
-        likely_typo_of = pkg_file_data.get("likely-typo-of")
-        arch_restriction = pkg_file_data.get("pkgfile-architecture-restriction")
+        path = ppf.path.path
+        likely_typo_of = ppf.expected_path
+        arch_restriction = ppf.architecture_restriction
         if likely_typo_of is not None:
             # Handles arch_restriction == 'all' at the same time due to how
             # the `likely-typo-of` is created
@@ -1047,7 +1195,7 @@ def _detect_misspelled_packaging_files(
                 )
             )
 
-        if not pkg_file_data.get("pkgfile-is-active-in-build", True):
+        if not ppf.definition.has_active_command:
             diagnostics.append(
                 Diagnostic(
                     diag_range,
@@ -1064,14 +1212,20 @@ def _detect_misspelled_packaging_files(
 
         if not explicit_package and name_segment is not None:
             basename = os.path.basename(path)
+            if basename == ppf.definition.stem:
+                continue
             alt_name = f"{binary_package}.{stem}"
             if arch_restriction is not None:
                 alt_name = f"{alt_name}.{arch_restriction}"
+            if ppf.definition.allow_name_segment:
+                or_alt_name = f' (or maybe "debian/{binary_package}.{basename}")'
+            else:
+                or_alt_name = ""
             diagnostics.append(
                 Diagnostic(
                     diag_range,
-                    f'Possible typo in "{path}". Consider renaming the file to "debian/{binary_package}.{basename}"'
-                    f' or "debian/{alt_name} if it is intended for {binary_package}',
+                    f'Possible typo in "{path}". Consider renaming the file to "debian/{alt_name}"'
+                    f"{or_alt_name} if it is intended for {binary_package}",
                     severity=DiagnosticSeverity.Warning,
                     source="debputy",
                     data=DiagnosticData(
